;**********************************************************************
;   								      *
;    Gestion de trois PWM                                             *
;    Precision de 256 increment                                       *
;    Robot qui avance !                                               *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Filename:	    PMItour.asm                                       *
;    Date:          12/10/99                                          *
;    File Version:  1                                                 *
;                                                                     *
;    Author:        Julien                                            *
;    Company:       Fribotte                                          *
;                                                                     * 
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required: rien                                             *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes: Quartz 4 Mhz                                              *
;                                                                     *
;                                                                     *
;**********************************************************************


	list      p=16F84             ; list directive to define processor
	#include <p16F84.inc>         ; processor specific variable definitions

	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_ON & _XT_OSC
; Pas de code protege, pas de watch hdog timer, delai de demarrage, horloge XT.


;**********************************************************************
;                                                                     *
;    Variables                                                        *
;                                                                     *
;                                                                     *
;**********************************************************************

;***** VARIABLE interruption
w_temp        	EQU     0x0C        	; variable used for context saving 
status_temp   	EQU     0x0D        	; variable used for context saving


;***** VARIABLE PWM
PwmGauche	EQU	0x10		; Valeur du PWM a appliquer entre 0 et 256 pour le moteur gauche
PwmDroit	EQU	0x11		; Valeur du PWM a appliquer entre 0 et 256 pour le moteur droit
PwmTourelle	EQU	0x12		; Valeur du PWM a appliquer entre 0 et 256 pour la tourelle
; Avance < 180 Par exemple 100. Recule >180, par exemple 250

;***** Etat de la tourelle
TrouveLumiere 	EQU	0x15		; la tourelle as-t-elle trouvee de la lumiere ou non ? 0 = non. 1 =oui.
SensTourelle	EQU	0x16		; 0 PWM decroissant, 1 PWM croissant


;***** VARIABLE TIMER
Phase		EQU	0x14		; valeur de la phase actuelle
					; Phase = 1 -> a 1 pour le moteur gauche , 0 les autres
					; Phase = 2 -> a 1 pour le moteur droit , 0 les autres
					; Phase = 3 -> a 1 pour la tourelle, 0 les autres
					; Phase = 0 -> a 0 pour tous les PWM


;**************************************************************** DEBUT
		ORG     0x000           ; processor reset vector
  		GOTO    main            ; go to beginning of program



;**********************************************************************
;                                                                     *
;    Interruption Timer                                               *
;                                                                     *
;                                                                     *
;**********************************************************************

		ORG     0x004           ; interrupt vector location

;***** Sauvegarde contexte
		MOVWF   w_temp          ; save off current W register contents
		MOVF	STATUS,0        ; move status register into W register
		MOVWF	status_temp     ; save off contents of STATUS register

;***** Test de la phase actuelle
		MOVF 	Phase,0		; On charge la phase dans W
		BTFSC	STATUS,2	; Resultat = 0 ? cad Phase = 0 ?
		GOTO	MoteurGaucheA1	; Bit =1, cad resultat =0, cad phase = 0, on passe en phase1
		DECF 	Phase,1		; On decroit la phase de 1		
		BTFSC	STATUS,2	; Resultat = 0 ? cad Phase = 1 ?
		GOTO	MoteurDroitA1	; Bit =1, cad resultat =0, cad phase = 1, on passe en phase2
		DECF 	Phase,1		; On decroit la phase de 1	
		BTFSC	STATUS,2	; Resultat = 0 ? cad Phase = 2 ?
		GOTO	TourelleA1	; Bit =1, cad resultat =0, cad phase = 2, on passe en phase3
		GOTO	PwmA0		; Sinon (phase = 3) on met tout a 0, on fait tout les tests et on passe en phase 0

;***** Quand le moteur gauche doit repasser a 1
MoteurGaucheA1
		MOVF	PwmGauche,0	; on recharge le PWM pour voir si il est nul
		BTFSC	STATUS,2	; Resultat = 0 ? cad PWM = 0 ?			
		GOTO	Aprestest1	; Bit =1, cad resultat =0, cad PWM = 0, on laisse le bit 3 du PORT A a 0 (moteur gauche)
		BSF	PORTA, 3	; Bit =0, cad resultat =1, cad PWM <> 0, on passe le bit 3 du PORT A a 1 (moteur gauche)

Aprestest1	CALL	Prescaleur8 	; On regle le prescaleur a 8

		MOVF	PwmGauche,0	; Nouveau reglage du timer
		SUBLW	0xFF		; On enleve 255 au PWM
		MOVWF	TMR0		; On le regle a la valeur PWM


		MOVLW	0x01		; On passe la Phase a 1
		MOVWF	Phase		;	/
		GOTO	FinInter

;***** Quand le moteur droit doit repasser a 1
MoteurDroitA1
		BCF	PORTA, 3	; On repasse le bit 3 du port A  a 0 (moteur gauche)

		MOVF	PwmDroit,0	; on recharge le PWM pour voir si il est nul
		BTFSC	STATUS,2	; Resultat = 0 ? cad PWM = 0 ?			
		GOTO	Aprestest2	; Bit =1, cad resultat =0, cad PWM = 0, on passe le bit 3 du PORT A a 0 (moteur gauche)
		BSF	PORTA, 2	; Bit =0, cad resultat =1, cad PWM <> 0, on passe le bit 3 du PORT A a 1 (moteur gauche)

Aprestest2	MOVF	PwmDroit,0	; Nouveau reglage du timer
		SUBLW	0xFF		; On enleve 255 au PWM
		MOVWF	TMR0		; On le regle a la valeur PWM

		MOVLW	0x02		; On passe la Phase a 2
		MOVWF	Phase		;	/
		GOTO	FinInter

;***** Quand La tourelle doit repasser a 1
TourelleA1
		BCF	PORTA, 2	; On repasse le bit 2 du port A  a 0 (moteur droit)
		BSF	PORTA, 1	; On repasse le bit 1 du port A  a 1 (tourelle)

		MOVF	PwmTourelle,0	; Nouveau reglage du timer
		SUBLW	0xFF		; On enleve 255 au PWM
		MOVWF	TMR0		; On le regle a la valeur PWM

		MOVLW	0x03		; On passe la Phase a 3
		MOVWF	Phase		;	/
		GOTO	FinInter

;***** Quand tout doit repasser a 0
PwmA0
		BCF	PORTA, 1	; On repasse le bit 1 du port A  a 0 (tourelle)

		CALL Prescaleur128	;***** On regle le prescaler a 128

		MOVLW	0x9A		; Nouveau reglage du timer a 154 (128*(256-154) = 13.056 ms en tout)
		MOVWF	TMR0		; 	/

		MOVLW	0x00		; On repasse la Phase a 0
		MOVWF	Phase		;	/

;***** On teste l'etat du capteur
		BTFSC	PORTA, 0	; On teste l'etat du bit 0 du port A (capteur)
		GOTO 	CapteurA1	; Il est a 1
		GOTO 	CapteurA0	; Il est a 0

;***** Le capteur est a 1
CapteurA1	INCF 	PwmTourelle,1	; On incremente le PWM de la tourelle
		BTFSS	STATUS,2	; PWM = 0 ?
		GOTO	TestSensA1	; Bit =0, cad resultat non nul, on continu			
		MOVLW	0x46		; Sinon on le repasse a 70
		MOVWF	PwmTourelle	;	/
		MOVLW	0x0		; Et le TrouveLumiere a 0
		MOVWF	TrouveLumiere	;	/
		GOTO	finTestcapteur	; on continu

TestSensA1	MOVF 	SensTourelle,0	
		BTFSS	STATUS,2	; SensTourelle = 0 ? cad qu'on decroissait le PWM
		GOTO	finTestcapteur	; Bit =0, cad resultat non nul, SensTourelle = 1, on a pas de changement de sens
		MOVLW	0x1		; on met le TrouveLumiere a 1
		MOVWF	TrouveLumiere	;	/
		MOVLW	0x1		; on met le SensTourelle a 1
		MOVWF	SensTourelle	;	/
		GOTO	finTestcapteur	; on continu


;***** Le capteur est a 0
CapteurA0	DECF 	PwmTourelle,1	; On decremente le PWM
		MOVF 	PwmTourelle,0	; On charge le PWM dans W
		SUBLW	0x46		; On y enleve 70
		BTFSS	STATUS,2	; Resultat = 0 ?	, cad le PWM = 70 
		GOTO	TestSensA0	; Bit =0, cad resultat non nul, on continu
		MOVLW	0xFF		; Sinon on le repasse a 255
		MOVWF	PwmTourelle	;	/
		MOVLW	0x0		; Et le TrouveLumiere a 0
		MOVWF	TrouveLumiere	;	/
		GOTO	finTestcapteur	; on continu

TestSensA0	MOVF 	SensTourelle,0	
		BTFSC	STATUS,2	; SensTourelle = 1 ? cad qu'on croissait le PWM
		GOTO	finTestcapteur	; Bit =1, cad resultat nul, SensTourelle = 0, on a pas de changement de sens
		MOVLW	0x1		; on met le TrouveLumiere a 1
		MOVWF	TrouveLumiere	;	/
		MOVLW	0x0		; on met le SensTourelle a 0
		MOVWF	SensTourelle	;	/
		GOTO	finTestcapteur	; on continu

finTestcapteur	NOP

;***** On regle les PWM des moteurs suivant le resultat de la tourelle

		MOVF 	TrouveLumiere,0	
		BTFSS	STATUS,2	; TrouveLumiere = 0 ? cad qu'on decroissait le PWM
		GOTO	RobotAvance		; Bit =0, cad resultat non nul, TrouveLumiere = 1, on fait avancer le robot
		GOTO 	RobotStop		; Sinon le robot reste a l'arret

RobotAvance	MOVF 	PwmTourelle,0	; On teste l'etat du PWM tourelle
		SUBLW	0xA0		; On lui enleve 160
		BTFSS	STATUS,0	; Carry = 0 ?	
		GOTO	DeuxiemeTest	; carry = 0, cad 160 - PWm <0 , cad PWM > 160
		GOTO	VersDroite	; carry = 1, cad 160 - PWm >=0 , cad PWM <= 160
DeuxiemeTest	MOVF 	PwmTourelle,0	; On teste l'etat du PWM tourelle a nouveau
		SUBLW	0xC8		; On lui enleve 200
		BTFSS	STATUS,0	; Carry = 0 ?	
		GOTO	VersGauche	; carry = 0, cad 200 - PWm <0 , cad PWM > 200
		GOTO	ToutDroit	; carry = 1, cad 200 - PWm >=0 , cad PWM <= 200

ToutDroit	MOVLW	0x64				; On met le PwmGauche a 100 (avance)
		MOVWF	PwmGauche				;	/
		MOVLW	0xFA				; On met le PwmDroit a 250 (avance)
		MOVWF	PwmDroit				;	/
		GOTO 	FinMouvement

VersGauche	MOVLW	0x64				; On met le PwmGauche a 100 (avance)
		MOVWF	PwmGauche				;	/
		MOVLW	0x64				; On met le PwmDroit a 100 (recule)
		MOVWF	PwmDroit				;	/
		GOTO 	FinMouvement

VersDroite	MOVLW	0xFA				; On met le PwmGauche a 250 (recule)
		MOVWF	PwmGauche				;	/
		MOVLW	0xFA				; On met le PwmDroit a 250 (avance)
		MOVWF	PwmDroit				;	/
		GOTO 	FinMouvement


RobotStop	MOVLW	0x0				; On met le PwmGauche a 0 (stop)
		MOVWF	PwmGauche				;	/
		MOVLW	0x0				; On met le PwmDroit a 0 (stop)
		MOVWF	PwmDroit				;	/

FinMouvement	NOP


;***** Redemarage interruption
FinInter	MOVF	TrouveLumiere,0	; on met le TrouveLumieresur le Port B
		MOVWF	PORTB		;	/
		
		BCF 	INTCON,	T0IF	; On efface l'interruption TimeR pour la relancer

;***** Restitution contexte
		MOVF    status_temp,0   ; retrieve copy of STATUS register
		MOVWF	STATUS          ; restore pre-isr STATUS register contents
		SWAPF   w_temp,f
		SWAPF   w_temp,w        ; restore pre-isr W register contents
		RETFIE                  ; return from interrupt



;**********************************************************************
;                                                                     *
;    Programme Main                                                   *
;                                                                     *
;                                                                     *
;**********************************************************************


main

;***** initialisations
	CALL	InitIO				; On intialise les ports d'E/S (A et B)
	MOVLW	0x0				; On met le PwmGauche a 0 (Stop)
	MOVWF	PwmGauche				;	/
	MOVLW	0x0				; On met le PwmDroit a 0 (Stop)
	MOVWF	PwmDroit				;	/
	MOVLW	0xB4				; On met le PwmTourelle a 180 (milieu)
	MOVWF	PwmTourelle				;	/
	MOVLW	0x0				; On met le TrouveLumiere a 0
	MOVWF	TrouveLumiere 			;	/
	MOVLW	0x0				; On met le SensTourelle a 0
	MOVWF	SensTourelle			;	/
	CALL 	InitRtcc			; On intialise l'interruption TimeR
	
boucle
	GOTO 	boucle				; Boucle infinie du main



;**********************************************************************
;                                                                     *
;    Initialisation des ports d'E/S                           	      *
;                                                                     *
;                                                                     *
;**********************************************************************


InitIO
	CLRF	PORTB		; Initialise le port B
	BSF	STATUS, RP0	; Bank 1 selectonnee
	MOVLW	0x0		; Port RB7-RB0 en sortie
	MOVWF	TRISB		;	/
	BCF 	STATUS,	RP0	; Bank 0 selectonnee

	CLRF 	PORTA		; Initialise le port A
	BSF 	STATUS,	RP0	; Bank 1 selectonnee
	MOVLW	0x01		; Port RA0 en entree RA1-RA3 en sortie
	MOVWF	TRISA		;	/
	BCF 	STATUS,	RP0	; Bank 0 selectonnee
	RETURN



;**********************************************************************
;                                                                     *
;    Initialisation de l'interruption TimeR                           *
;                                                                     *
;                                                                     *
;**********************************************************************


InitRtcc

;*****  On regle le prescaler
	CLRWDT			; Clear WDT and prescaler
	BSF	STATUS, RP0	; Bank 1 selectonnee
	MOVLW	B'10000010'	; Timer interne, prescaler sur Timer, Prescaler a 8.
	MOVWF	OPTION_REG	;	/
	BCF	STATUS, RP0	; Bank 0 selectonnee

;*****  On regle l'interruption

	MOVLW	0xFA		; Initialisation de la valeur du TimeR
	MOVWF	TMR0		;	/

	MOVLW   B'10100000'     ; enable interrupts
	MOVWF   INTCON          ;       /
	RETURN

;**********************************************************************
;                                                                     *
;    Reglage du Prescaleur sur 8	                              *
;                                                                     *
;                                                                     *
;**********************************************************************

Prescaleur8

;***** On regle le prescaler a 8
		BSF	STATUS, RP0	; Bank 1 selectonnee
		MOVLW	B'10000010'	; Timer interne, prescaler sur Timer, Prescaler a 8.
		MOVWF	OPTION_REG	;	/
		BCF	STATUS, RP0	; Bank 0 selectonnee

	RETURN

;**********************************************************************
;                                                                     *
;    Reglage du Prescaleur sur 128	                              *
;                                                                     *
;                                                                     *
;**********************************************************************

Prescaleur128

;***** On regle le prescaler a 128
		BSF	STATUS, RP0	; Bank 1 selectonnee
		MOVLW	B'10000110'	; Timer interne, prescaler sur Timer, Prescaler a 128.
		MOVWF	OPTION_REG	;	/
		BCF	STATUS, RP0	; Bank 0 selectonnee

	RETURN

;**********************************************************************
;                                                                     *
;    Fin				                              *
;                                                                     *
;                                                                     *
;**********************************************************************


	END                     ; directive 'end of program'


